@@ -4,16 +4,21 @@ use warnings;
 use utf8;
 
 package Neo4j::Driver::Net::Bolt;
-# ABSTRACT: Networking delegate for Neo4j Bolt
-$Neo4j::Driver::Net::Bolt::VERSION = '0.24'; # TRIAL
+# ABSTRACT: Network controller for Neo4j Bolt
+$Neo4j::Driver::Net::Bolt::VERSION = '0.37'; # TRIAL
+
+# This package is not part of the public Neo4j::Driver API.
+
 
 use Carp qw(croak);
 our @CARP_NOT = qw(Neo4j::Driver::Transaction Neo4j::Driver::Transaction::Bolt);
 
+use Try::Tiny;
 use URI 1.25;
 
 use Neo4j::Driver::Result::Bolt;
 use Neo4j::Driver::ServerInfo;
+use Neo4j::Error;
 
 
 # Neo4j::Bolt < 0.10 didn't report human-readable error messages
@@ -33,6 +38,8 @@ sub new {
 	# uncoverable pod
 	my ($class, $driver) = @_;
 	
+	croak "Concurrent transactions are unsupported in Bolt; use multiple sessions" if $driver->{concurrent_tx};
+	
 	my $uri = $driver->{uri};
 	if ($driver->{auth}) {
 		croak "Only Basic Authentication is supported" if $driver->{auth}->{scheme} ne 'basic';
@@ -40,13 +47,11 @@ sub new {
 		$uri->userinfo( $driver->{auth}->{principal} . ':' . $driver->{auth}->{credentials} );
 	}
 	
-	my $protocol = "Bolt";
 	my $net_module = $driver->{net_module} || 'Neo4j::Bolt';
 	if ($net_module eq 'Neo4j::Bolt') {
 		croak "Protocol scheme 'bolt' is not supported (Neo4j::Bolt not installed)\n"
 			. "Neo4j::Driver will support 'bolt' URLs if the Neo4j::Bolt module is installed.\n"
 			unless eval { require Neo4j::Bolt; 1 };
-		$protocol = "Bolt/1.0" if $Neo4j::Bolt::VERSION le "0.20";
 	}
 	
 	my $cxn;
@@ -59,24 +64,67 @@ sub new {
 	else {
 		$cxn = $net_module->connect( "$uri", $driver->{http_timeout} );
 	}
-	croak $class->_bolt_error($cxn) unless $cxn->connected;
-	$protocol = "Bolt/" . $cxn->protocol_version if $cxn->can('protocol_version');
+	$class->_trigger_bolt_error( $cxn, $driver->{plugins} ) unless $cxn->connected;
 	
 	return bless {
 		net_module => $net_module,
 		connection => $cxn,
+		uri => $uri,
 		result_module => $net_module->can('result_handlers') ? ($net_module->result_handlers)[0] : $RESULT_MODULE,
-		server_info => Neo4j::Driver::ServerInfo->new({
-			uri => $uri,
-			version => $cxn->server_id,
-			protocol => $protocol,
-		}),
+		server_info => $driver->{server_info},
 		cypher_types => $driver->{cypher_types},
 		active_tx => 0,
 	}, $class;
 }
 
 
+# Trigger an error using the given event handler.
+# Meant to only be called after a failure has occurred.
+# May also be called as class method.
+# $ref may be a Neo4j::Bolt ResultStream, Cxn, Txn.
+# $error_handler may be a coderef or the event manager.
+sub _trigger_bolt_error {
+	my ($self, $ref, $error_handler, $connection) = @_;
+	my $error = 'Neo4j::Error';
+	
+	$error = $error->append_new( Server => {
+		code => scalar $ref->server_errcode,
+		message => scalar $ref->server_errmsg,
+		raw => scalar try { $ref->get_failure_details },  # Neo4j::Bolt >= 0.41
+	}) if try { $ref->server_errcode || $ref->server_errmsg };
+	
+	$error = $error->append_new( Network => {
+		code => scalar $ref->client_errnum,
+		message => scalar $ref->client_errmsg // $BOLT_ERROR{$ref->client_errnum},
+		as_string => $self->_bolt_error($ref),
+	}) if try { $ref->client_errnum || $ref->client_errmsg };
+	
+	$error = $error->append_new( Network => {
+		code => scalar $ref->errnum,
+		message => scalar $ref->errmsg // $BOLT_ERROR{$ref->errnum},
+		as_string => $self->_bolt_error($ref),
+	}) if try { $ref->errnum || $ref->errmsg };
+	
+	try {
+		my $cxn = $connection // $self->{connection};
+		$error = $error->append_new( Network => {
+			code => scalar $cxn->errnum,
+			message => scalar $cxn->errmsg // $BOLT_ERROR{$cxn->errnum},
+			as_string => $self->_bolt_error($cxn),
+		}) if try { $cxn->errnum || $cxn->errmsg } && $cxn != $ref;
+		$cxn->reset_cxn;
+		$error = $error->append_new( Internal => {  # perlbolt#51
+			code => scalar $cxn->errnum,
+			message => scalar $cxn->errmsg // $BOLT_ERROR{$cxn->errnum},
+			as_string => $self->_bolt_error($cxn),
+		}) if try { $cxn->errnum || $cxn->errmsg };
+	};
+	
+	return $error_handler->($error) if ref $error_handler eq 'CODE';
+	$error_handler->trigger(error => $error);
+}
+
+
 sub _bolt_error {
 	my (undef, $ref) = @_;
 	
@@ -92,7 +140,13 @@ sub _bolt_error {
 
 sub _server {
 	my ($self) = @_;
-	return $self->{server_info};
+	
+	my $cxn = $self->{connection};
+	return $self->{server_info} = Neo4j::Driver::ServerInfo->new({
+		uri => $self->{uri},
+		version => $cxn->server_id,
+		protocol => $cxn->can('protocol_version') ? $cxn->protocol_version : '',
+	});
 }
 
 
@@ -122,35 +176,13 @@ sub _run {
 	if ($statement->[0]) {
 		$stream = $query_runner->run_query( @$statement, $self->{database} );
 		
-		if (! $stream) {
-			$tx->{closed} = 1;
-			$self->{active_tx} = 0;
-			croak $self->_bolt_error( $self->{connection} );
-		}
-		if ($stream->failure) {
+		if (! $stream || $stream->failure) {
 			# failure() == -1 is an error condition because run_query_()
 			# always calls update_errstate_rs_obj()
 			
-			if ( ! $stream->server_errcode && ! $stream->server_errmsg ) {
-				$tx->{closed} = 1;
-				$self->{active_tx} = 0;
-				croak $self->_bolt_error( $stream );
-			}
-			
-			# <https://neo4j.com/docs/status-codes/4.2/> suggests that
-			# transactions should already have been rolled back and
-			# closed automatically at this point due to the server error.
-			# This is usually what happens on HTTP (but see neo4j#12651).
-			# However, on Bolt, the transaction tends to remain open
-			# (albeit marked as failed, thus uncommittable). Just
-			# attempting an explicit rollback whenever the Neo4j server
-			# reports any errors should fix that. If there are additional
-			# errors during the rollback, those must be ignored.
-			eval { $tx->{failed} = 1; $tx->rollback; } unless $tx->{failed};
 			$tx->{closed} = 1;
 			$self->{active_tx} = 0;
-			
-			croak sprintf "%s:\n%s\n%s", $stream->server_errcode, $stream->server_errmsg, $self->_bolt_error( $stream );
+			$self->_trigger_bolt_error( $stream, $tx->{error_handler} );
 		}
 		
 		$result = $self->{result_module}->new({
@@ -159,6 +191,7 @@ sub _run {
 			statement => $statement_json,
 			cypher_types => $self->{cypher_types},
 			server_info => $self->{server_info},
+			error_handler => $tx->{error_handler},
 		});
 	}
 	
@@ -167,45 +200,15 @@ sub _run {
 
 
 sub _new_tx {
-	my ($self) = @_;
+	my ($self, $driver_tx) = @_;
+	
+	my $params = {};
+	$params->{mode} = lc substr $driver_tx->{mode}, 0, 1 if $driver_tx->{mode};
 	
 	my $transaction = "$self->{net_module}::Txn";
 	return unless $transaction->can('new');
-	return $transaction->new( $self->{connection} );
+	return $transaction->new( $self->{connection}, $params, $self->{database} );
 }
 
 
 1;
-
-__END__
-
-=pod
-
-=encoding UTF-8
-
-=head1 NAME
-
-Neo4j::Driver::Net::Bolt - Networking delegate for Neo4j Bolt
-
-=head1 VERSION
-
-version 0.24
-
-=head1 DESCRIPTION
-
-The L<Neo4j::Driver::Net::Bolt> package is not part of the
-public L<Neo4j::Driver> API.
-
-=head1 AUTHOR
-
-Arne Johannessen <ajnn@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
-
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
-
-=cut
