@@ -4,10 +4,13 @@ use warnings;
 use utf8;
 
 package Neo4j::Driver::Net::HTTP;
-# ABSTRACT: Networking delegate for Neo4j HTTP
-$Neo4j::Driver::Net::HTTP::VERSION = '0.24'; # TRIAL
+# ABSTRACT: Network controller for Neo4j HTTP
+$Neo4j::Driver::Net::HTTP::VERSION = '0.37'; # TRIAL
 
-use Carp qw(croak);
+# This package is not part of the public Neo4j::Driver API.
+
+
+use Carp qw(carp croak);
 our @CARP_NOT = qw(Neo4j::Driver::Transaction Neo4j::Driver::Transaction::HTTP);
 
 use Time::Piece 1.20 qw();
@@ -33,13 +36,19 @@ sub new {
 	# uncoverable pod
 	my ($class, $driver) = @_;
 	
-	my $net_module = $driver->{net_module} || 'Neo4j::Driver::Net::HTTP::LWP';
+	$driver->{plugins}->{default_handlers}->{http_adapter_factory} //= sub {
+		my $net_module = $driver->{net_module} || 'Neo4j::Driver::Net::HTTP::LWP';
+		return $net_module->new($driver);
+	};
+	my $http_adapter = $driver->{plugins}->trigger('http_adapter_factory', $driver);
 	
 	my $self = bless {
-		die_on_error => $driver->{die_on_error},
+		events => $driver->{plugins},
 		cypher_types => $driver->{cypher_types},
-		http_agent => $net_module->new($driver),
+		server_info => $driver->{server_info},
+		http_agent => $http_adapter,
 		want_jolt => $driver->{jolt},
+		want_concurrent => $driver->{concurrent_tx} // 1,
 		active_tx => {},
 	}, $class;
 	
@@ -52,12 +61,14 @@ sub new {
 sub _server {
 	my ($self) = @_;
 	
-	return $self->{server_info} if exists $self->{server_info};
-	
 	my ($neo4j_version, $tx_endpoint);
 	my @discovery_queue = ($DISCOVERY_ENDPOINT);
 	while (@discovery_queue) {
-		my $tx = { transaction_endpoint => shift @discovery_queue };
+		my $events = $self->{events};
+		my $tx = {
+			error_handler => sub { $events->trigger(error => shift) },
+			transaction_endpoint => shift @discovery_queue,
+		};
 		my $service = $self->_request($tx, 'GET')->_json;
 		
 		$neo4j_version = $service->{neo4j_version};
@@ -79,15 +90,10 @@ sub _server {
 	$self->{server_info} = Neo4j::Driver::ServerInfo->new({
 		uri => $self->{http_agent}->uri,
 		version => "Neo4j/$neo4j_version",
-		protocol => $self->{http_agent}->protocol,
 		time_diff => Time::Piece->new - $date,
+		tx_endpoint => $tx_endpoint,
 	});
 	
-	$self->{endpoints} = {
-		new_transaction => "$tx_endpoint",
-		new_commit => "$tx_endpoint/$COMMIT_ENDPOINT",
-	} if $tx_endpoint;
-	
 	return $self->{server_info};
 }
 
@@ -96,6 +102,12 @@ sub _server {
 sub _set_database {
 	my ($self, $database) = @_;
 	
+	my $tx_endpoint = $self->{server_info}->{tx_endpoint};
+	$self->{endpoints} = {
+		new_transaction => "$tx_endpoint",
+		new_commit => "$tx_endpoint/$COMMIT_ENDPOINT",
+	} if $tx_endpoint;
+	
 	return unless defined $database;
 	$database = URI::Escape::uri_escape_utf8 $database;
 	$self->{endpoints}->{new_transaction} =~ s/\{databaseName}/$database/;
@@ -107,6 +119,12 @@ sub _set_database {
 sub _run {
 	my ($self, $tx, @statements) = @_;
 	
+	if ( ! $self->{want_concurrent} ) {
+		my $is_concurrent = %{$self->{active_tx}} && ! defined $tx->{commit_endpoint};
+		$is_concurrent ||= keys %{$self->{active_tx}} > 1;
+		$is_concurrent and carp "Concurrent transactions for HTTP are disabled; use multiple sessions or enable the concurrent_tx config option (this warning may become fatal in a future Neo4j::Driver version)";
+	}
+	
 	my $json = { statements => \@statements };
 	return $self->_request($tx, 'POST', $json)->_results;
 }
@@ -117,8 +135,12 @@ sub _run {
 sub _accept_for {
 	my ($self, $method) = @_;
 	
+	$self->{want_jolt} = 'v1' if ! defined $self->{want_jolt}
+		&& $self->{server_info} && $self->{server_info}->{version} =~ m{^Neo4j/4\.[234]\.};
+	
 	# GET requests may fail if Neo4j sees clients that support Jolt, see neo4j #12644
-	my @modules = ( $self->{http_agent}->result_handlers, @RESULT_MODULES );
+	my @modules = @RESULT_MODULES;
+	unshift @modules, $self->{http_agent}->result_handlers if $self->{http_agent}->can('result_handlers');
 	my @accept = map { $_->_accept_header( $self->{want_jolt}, $method ) } @modules;
 	return $self->{accept_for}->{$method} = join ', ', @accept;
 }
@@ -129,7 +151,8 @@ sub _accept_for {
 sub _result_module_for {
 	my ($self, $content_type) = @_;
 	
-	my @modules = ( $self->{http_agent}->result_handlers, @RESULT_MODULES );
+	my @modules = @RESULT_MODULES;
+	unshift @modules, $self->{http_agent}->result_handlers if $self->{http_agent}->can('result_handlers');
 	foreach my $module (@modules) {
 		if ($module->_acceptable($content_type)) {
 			return $self->{result_module_for}->{$content_type} = $module;
@@ -151,7 +174,7 @@ sub _request {
 	my $accept = $self->{accept_for}->{$method}
 	             // $self->_accept_for($method);
 	
-	$self->{http_agent}->request($method, $tx_endpoint, $json, $accept);
+	$self->{http_agent}->request($method, $tx_endpoint, $json, $accept, $tx->{mode});
 	
 	my $header = $self->{http_agent}->http_header;
 	$tx->{closed} = $header->{success};  # see _parse_tx_status() and neo4j #12651
@@ -163,7 +186,7 @@ sub _request {
 		http_method => $method,
 		http_path => $tx_endpoint,
 		http_header => $header,
-		die_on_error => $self->{die_on_error},
+		error_handler => $tx->{error_handler},
 		cypher_types => $self->{cypher_types},
 		server_info => $self->{server_info},
 		statements => $json ? $json->{statements} : [],
@@ -217,38 +240,4 @@ sub _is_active_tx {
 }
 
 
-
 1;
-
-__END__
-
-=pod
-
-=encoding UTF-8
-
-=head1 NAME
-
-Neo4j::Driver::Net::HTTP - Networking delegate for Neo4j HTTP
-
-=head1 VERSION
-
-version 0.24
-
-=head1 DESCRIPTION
-
-The L<Neo4j::Driver::Net::HTTP> package is not part of the
-public L<Neo4j::Driver> API.
-
-=head1 AUTHOR
-
-Arne Johannessen <ajnn@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
-
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
-
-=cut
