@@ -5,7 +5,7 @@ use utf8;
 
 package Neo4j::Driver::Result;
 # ABSTRACT: Result of running a Cypher statement (a stream of records)
-$Neo4j::Driver::Result::VERSION = '0.24'; # TRIAL
+$Neo4j::Driver::Result::VERSION = '0.37'; # TRIAL
 
 use parent 'Neo4j::Driver::StatementResult';
 
@@ -20,6 +20,7 @@ our $fake_attached = 0;  # 1: simulate an attached stream (only used in testing)
 
 
 sub new {
+	# uncoverable pod (private method)
 	my ($class) = @_;
 	
 	return bless { buffer => [] }, $class;
@@ -37,11 +38,7 @@ sub _column_keys {
 sub keys {
 	my ($self) = @_;
 	
-	# Don't break encapsulation by just returning the original reference
-	# because ResultColumns depends on the {columns} field being intact.
-	my @keys = ();
-	@keys = @{ $self->{result}->{columns} } if $self->{result}->{columns};
-	return wantarray ? @keys : [@keys];
+	return @{ $self->{result}->{columns} };
 }
 
 
@@ -139,6 +136,7 @@ sub fetch {
 
 
 sub peek {
+	# uncoverable pod (experimental feature)
 	my ($self) = @_;
 	
 	croak "iterator is exhausted" if $self->{exhausted};
@@ -157,20 +155,25 @@ sub has_next {
 
 
 sub attached {
+	# uncoverable pod (see Deprecations.pod)
 	my ($self) = @_;
 	
+	warnings::warnif deprecated => __PACKAGE__ . "->attached is deprecated";
 	return $self->{attached};
 }
 
 
 sub detach {
+	# uncoverable pod (see Deprecations.pod)
 	my ($self) = @_;
 	
+	warnings::warnif deprecated => __PACKAGE__ . "->detach is deprecated";
 	return $self->_fill_buffer;
 }
 
 
 sub consume {
+	# uncoverable pod (experimental feature)
 	my ($self) = @_;
 	
 	# Neo4j::Bolt doesn't offer direct access to neo4j_close_results()
@@ -186,11 +189,12 @@ sub summary {
 	
 	$self->{summary} //= Neo4j::Driver::ResultSummary->new( $self->{result}, $self->{notifications}, $self->{statement}, $self->{server_info} );
 	
-	return $self->{summary}->init;
+	return $self->{summary}->_init;
 }
 
 
 sub stats {
+	# uncoverable pod (see Deprecations.pod)
 	my ($self) = @_;
 	warnings::warnif deprecated => __PACKAGE__ . "->stats is deprecated; use summary instead";
 	
@@ -213,7 +217,7 @@ Neo4j::Driver::Result - Result of running a Cypher statement (a stream of record
 
 =head1 VERSION
 
-version 0.24
+version 0.37
 
 =head1 SYNOPSIS
 
@@ -258,6 +262,8 @@ an explicit transaction) until the transaction is closed, whichever
 comes first. When a result stream has become invalid I<before> it
 was detached, calling any methods in this class may fail.
 
+To obtain a query result, call L<Neo4j::Driver::Transaction/"run">.
+
 Until version 0.18, this module was named C<StatementResult>.
 
 =head1 METHODS
@@ -352,25 +358,12 @@ depend upon these features.
 
 =head2 Calling in scalar context
 
- $keys = $result->keys;  # arrayref
+ $count = $result->keys;
 
-The C<keys()> method returns an array reference if called in scalar
+The C<keys()> method returns the number of columns if called in scalar
 context.
 
-=head2 Control result stream attachment
-
- $buffered = $result->attached;  # boolean
- $count = $result->detach;  # number of records fetched
-
-If necessary, C<detach()> can force the entire result stream to
-be buffered locally, so that it will be available to C<fetch()>
-indefinitely, irrespective of other statements run on the same
-session. Essentially, the outcome is the same as calling C<list()>,
-except that C<fetch()> can continue to be used because the result
-is not exhausted.
-
-Most of the official drivers do not offer these methods. Their
-usefulness is doubtful. They may be removed in future versions.
+Until version 0.25, it returned an array reference instead.
 
 =head2 Discarding the result stream
 
@@ -386,6 +379,9 @@ As a side effect, discarding the result yields a summary of it.
 
  $result_summary = $result->consume;
 
+Using a result after this method has been called is discouraged.
+This may become a fatal error in future versions.
+
 All of the official drivers offer this method, but it doesn't appear
 to be necessary here, since L<Neo4j::Bolt::ResultStream> reliably
 calls C<neo4j_close_results()> in its C<DESTROY()> method. It may
@@ -410,10 +406,10 @@ buffer and detach the result, but will never exhaust it.
 L<Neo4j::Driver::B<ResultSummary>>
 
 =item * Equivalent documentation for the official Neo4j drivers:
-L<Result (Java)|https://neo4j.com/docs/api/java-driver/current/index.html?org/neo4j/driver/Result.html>,
-L<Result (Python)|https://neo4j.com/docs/api/python-driver/current/api.html#result>,
-L<Result (JavaScript)|https://neo4j.com/docs/api/javascript-driver/current/class/src/result.js~Result.html>,
-L<IResult (.NET)|https://neo4j.com/docs/api/dotnet-driver/4.0/html/f1ac31ec-c6dd-798b-b5d6-3ca0794d7502.htm>
+L<Result (Java)|https://neo4j.com/docs/api/java-driver/5.2/org.neo4j.driver/org/neo4j/driver/Result.html>,
+L<Result (Python)|https://neo4j.com/docs/api/python-driver/5.2/api.html#result>,
+L<Result (JavaScript)|https://neo4j.com/docs/api/javascript-driver/5.2/class/lib6/result.js~Result.html>,
+L<IResult (.NET)|https://neo4j.com/docs/api/dotnet-driver/5.2/html/f1ac31ec-c6dd-798b-b5d6-3ca0794d7502.htm>
 
 =back
 
@@ -421,12 +417,15 @@ L<IResult (.NET)|https://neo4j.com/docs/api/dotnet-driver/4.0/html/f1ac31ec-c6dd
 
 Arne Johannessen <ajnn@cpan.org>
 
+If you contact me by email, please make sure you include the word
+"Perl" in your subject header to help beat the spam filters.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
+This software is Copyright (c) 2016-2023 by Arne Johannessen.
 
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
+This is free software; you can redistribute it and/or modify it under
+the terms of the Artistic License 2.0 or (at your option) the same terms
+as the Perl 5 programming language system itself.
 
 =cut
