@@ -5,17 +5,24 @@ use utf8;
 
 package Neo4j::Driver::Result::Bolt;
 # ABSTRACT: Bolt result handler
-$Neo4j::Driver::Result::Bolt::VERSION = '0.24'; # TRIAL
+$Neo4j::Driver::Result::Bolt::VERSION = '0.37'; # TRIAL
+
+# This package is not part of the public Neo4j::Driver API.
+
 
 use parent 'Neo4j::Driver::Result';
 
 use Carp qw(croak);
+our @CARP_NOT = qw(Neo4j::Driver::Net::Bolt Neo4j::Driver::Result);
+
+use Neo4j::Driver::Net::Bolt;
 
 
 our $gather_results = 0;  # 1: detach from the stream immediately (yields JSON-style result; used for testing)
 
 
 sub new {
+	# uncoverable pod (private method)
 	my ($class, $params) = @_;
 	
 	# Holding a reference to the Bolt connection is important, because
@@ -35,6 +42,7 @@ sub new {
 		cxn => $params->{bolt_connection},  # important to avoid dereferencing the connection
 		stream => $params->{bolt_stream},
 		server_info => $params->{server_info},
+		error_handler => $params->{error_handler},
 	};
 	bless $self, $class;
 	
@@ -56,13 +64,13 @@ sub _gather_results {
 	while ( my @row = $stream->fetch_next ) {
 		
 		croak 'next true and failure/success mismatch: ' . $stream->failure . '/' . $stream->success unless $stream->failure == -1 || $stream->success == -1 || ($stream->failure xor $stream->success);  # assertion
-		croak 'next true and error: client ' . $stream->client_errnum . ' ' . $stream->client_errmsg . '; server ' . $stream->server_errcode . ' ' . $stream->server_errmsg if $stream->failure && $stream->failure != -1;
+		Neo4j::Driver::Net::Bolt->_trigger_bolt_error( $stream, $self->{error_handler}, $self->{cxn} ) if $stream->failure && $stream->failure != -1;
 		
 		push @data, { row => \@row, meta => [] };
 	}
 	
 	croak 'next false and failure/success mismatch: ' . $stream->failure . '/' . $stream->success unless  $stream->failure == -1 || $stream->success == -1 || ($stream->failure xor $stream->success);  # assertion
-	croak 'next false and error: client ' . $stream->client_errnum . ' ' . $stream->client_errmsg . '; server ' . $stream->server_errcode . ' ' . $stream->server_errmsg if $stream->failure && $stream->failure != -1;
+	Neo4j::Driver::Net::Bolt->_trigger_bolt_error( $stream, $self->{error_handler}, $self->{cxn} ) if $stream->failure && $stream->failure != -1;
 	
 	$self->{stream} = undef;
 	$self->{cxn} = undef;
@@ -87,9 +95,7 @@ sub _fetch_next {
 	unless ($self->{stream}->success) {
 		# success() == -1 is not an error condition; it simply
 		# means that there are no more records on the stream
-		my $stream = $self->{stream};
-		croak sprintf "Bolt error %i: %s", $stream->client_errnum, $stream->client_errmsg unless $stream->server_errcode || $stream->server_errmsg;
-		croak sprintf "%s:\n%s\nBolt error %i: %s", $stream->server_errcode, $stream->server_errmsg, $stream->client_errnum, $stream->client_errmsg;
+		Neo4j::Driver::Net::Bolt->_trigger_bolt_error( $self->{stream}, $self->{error_handler}, $self->{cxn} );
 	}
 	
 	return $self->_init_record( $record );
@@ -190,46 +196,3 @@ sub _deep_bless {
 
 
 1;
-
-__END__
-
-=pod
-
-=encoding UTF-8
-
-=head1 NAME
-
-Neo4j::Driver::Result::Bolt - Bolt result handler
-
-=head1 VERSION
-
-version 0.24
-
-=head1 DESCRIPTION
-
-The L<Neo4j::Driver::Result::Bolt> package is not part of the
-public L<Neo4j::Driver> API.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Neo4j::Driver::Net>
-
-=item * L<Neo4j::Driver::Result>
-
-=back
-
-=head1 AUTHOR
-
-Arne Johannessen <ajnn@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
-
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
-
-=cut
