@@ -5,7 +5,10 @@ use utf8;
 
 package Neo4j::Driver::Result::JSON;
 # ABSTRACT: JSON/REST result handler
-$Neo4j::Driver::Result::JSON::VERSION = '0.24'; # TRIAL
+$Neo4j::Driver::Result::JSON::VERSION = '0.37'; # TRIAL
+
+# This package is not part of the public Neo4j::Driver API.
+
 
 use parent 'Neo4j::Driver::Result';
 
@@ -13,9 +16,12 @@ use Carp qw(carp croak);
 our @CARP_NOT = qw(Neo4j::Driver::Net::HTTP);
 use Try::Tiny;
 
-use JSON::MaybeXS 1.003003 ();
 use URI 1.31;
 
+use Neo4j::Error;
+
+
+my ($TRUE, $FALSE);
 
 my $MEDIA_TYPE = "application/json";
 my $ACCEPT_HEADER = "$MEDIA_TYPE";
@@ -23,8 +29,11 @@ my $ACCEPT_HEADER_POST = "$MEDIA_TYPE;q=0.5";
 
 
 sub new {
+	# uncoverable pod (private method)
 	my ($class, $params) = @_;
 	
+	($TRUE, $FALSE) = @{ $params->{http_agent}->json_coder->decode('[true,false]') } unless $TRUE;
+	
 	my $json = $class->_parse_json($params);
 	
 	my @results = ();
@@ -76,32 +85,33 @@ sub _parse_json {
 	my (undef, $params) = @_;
 	
 	my $response = $params->{http_agent}->fetch_all;
-	
-	my @errors = ();
-	if (! $params->{http_header}->{success}) {
-		my $reason_phrase = $params->{http_agent}->http_reason;
-		push @errors, "HTTP error: $params->{http_header}->{status} $reason_phrase on $params->{http_method} to $params->{http_path}";
-	}
-	
+	my $error = 'Neo4j::Error';
 	my $json;
 	try {
 		$json = $params->{http_agent}->json_coder->decode($response);
 	}
 	catch {
-		push @errors, $_;
-		$json = {};
+		$error = $error->append_new( Internal => {
+			as_string => "$_",
+			raw => $response,
+		});
 	};
 	if (ref $json->{errors} eq 'ARRAY') {
-		foreach my $error (@{$json->{errors}}) {
-			$error = "$error->{code}: $error->{message}" if ref $error eq 'HASH';
-			push @errors, $error;
-		}
+		$error = $error->append_new( Server => $_ ) for @{$json->{errors}};
+	}
+	if ($json->{message}) {
+		$error = $error->append_new( Internal => $json->{message} );
+		# can happen when the Jersey ServletContainer intercepts the request
+	}
+	if (! $params->{http_header}->{success}) {
+		$error = $error->append_new( Network => {
+			code => $params->{http_header}->{status},
+			as_string => sprintf( "HTTP error: %s %s on %s to %s",
+				$params->{http_header}->{status}, $params->{http_agent}->http_reason, $params->{http_method}, $params->{http_path} ),
+		});
 	}
 	
-	if (@errors) {
-		croak join "\n", @errors if $params->{die_on_error};
-		carp join "\n", @errors;
-	}
+	$params->{error_handler}->($error) if ref $error;
 	
 	return $json;
 }
@@ -224,7 +234,7 @@ sub _deep_bless {
 	if (ref $data eq '' && ref $rest eq '') {  # scalar
 		return $data;
 	}
-	if ( JSON::MaybeXS::is_bool($data) && JSON::MaybeXS::is_bool($rest) ) {  # boolean
+	if ( $data == $TRUE && $rest == $TRUE || $data == $FALSE && $rest == $FALSE ) {  # boolean
 		return $data;
 	}
 	
@@ -237,8 +247,9 @@ sub _deep_bless {
 sub _accept_header {
 	my (undef, $want_jolt, $method) = @_;
 	
+	# 'v1' is used as an internal marker for Neo4j 4
 	# Note: Neo4j < 4.2 doesn't fail gracefully if Jolt is the only acceptable response type.
-	return if $want_jolt;
+	return if $want_jolt && $want_jolt ne 'v1';
 	
 	return ($ACCEPT_HEADER_POST) if $method eq 'POST';
 	return ($ACCEPT_HEADER);
@@ -254,46 +265,3 @@ sub _acceptable {
 
 
 1;
-
-__END__
-
-=pod
-
-=encoding UTF-8
-
-=head1 NAME
-
-Neo4j::Driver::Result::JSON - JSON/REST result handler
-
-=head1 VERSION
-
-version 0.24
-
-=head1 DESCRIPTION
-
-The L<Neo4j::Driver::Result::JSON> package is not part of the
-public L<Neo4j::Driver> API.
-
-=head1 SEE ALSO
-
-=over
-
-=item * L<Neo4j::Driver::Net>
-
-=item * L<Neo4j::Driver::Result>
-
-=back
-
-=head1 AUTHOR
-
-Arne Johannessen <ajnn@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
-
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
-
-=cut
