@@ -5,13 +5,14 @@ use utf8;
 
 package Neo4j::Driver::Transaction;
 # ABSTRACT: Logical container for an atomic unit of work
-$Neo4j::Driver::Transaction::VERSION = '0.24'; # TRIAL
+$Neo4j::Driver::Transaction::VERSION = '0.37'; # TRIAL
 
 use Carp qw(croak);
 our @CARP_NOT = qw(
 	Neo4j::Driver::Session
 	Neo4j::Driver::Session::Bolt
 	Neo4j::Driver::Session::HTTP
+	Try::Tiny
 );
 use Scalar::Util qw(blessed);
 
@@ -19,15 +20,19 @@ use Neo4j::Driver::Result;
 
 
 sub new {
-	my ($class, $session) = @_;
+	# uncoverable pod (private method)
+	my ($class, $session, $mode) = @_;
 	
+	my $events = $session->{driver}->{plugins};
 	my $transaction = {
-		cypher_filter => $session->{driver}->{cypher_filter},
+		cypher_params_v2 => $session->{cypher_params_v2},
 		net => $session->{net},
+		mode => $mode,
 		unused => 1,  # for HTTP only
 		closed => 0,
 		return_graph => 0,
 		return_stats => 1,
+		error_handler => sub { $events->trigger(error => shift) },
 	};
 	
 	return bless $transaction, $class;
@@ -39,8 +44,11 @@ sub run {
 	
 	croak 'Transaction already closed' unless $self->is_open;
 	
+	warnings::warnif deprecated => __PACKAGE__ . "->{return_graph} is deprecated" if $self->{return_graph};
+	
 	my @statements;
 	if (ref $query eq 'ARRAY') {
+		warnings::warnif deprecated => "run() with multiple statements is deprecated";
 		foreach my $args (@$query) {
 			push @statements, $self->_prepare(@$args);
 		}
@@ -56,6 +64,7 @@ sub run {
 	
 	if (scalar @statements <= 1) {
 		my $result = $results[0] // Neo4j::Driver::Result->new;
+		warnings::warnif deprecated => "run() in list context is deprecated" if wantarray;
 		return wantarray ? $result->list : $result;
 	}
 	return wantarray ? @results : \@results;
@@ -81,13 +90,10 @@ sub _prepare {
 		$params = {@parameters};
 	}
 	
-	if ($self->{cypher_filter}) {
-		croak "Unimplemented cypher filter '$self->{cypher_filter}'" if $self->{cypher_filter} ne 'params';
-		if (defined $params) {
-			my @params_quoted = map {quotemeta} keys %$params;
-			my $params_re = join '|', @params_quoted, map {"`$_`"} @params_quoted;
-			$query =~ s/\{($params_re)}/\$$1/g;
-		}
+	if ($self->{cypher_params_v2} && defined $params) {
+		my @params_quoted = map {quotemeta} keys %$params;
+		my $params_re = join '|', @params_quoted, map {"`$_`"} @params_quoted;
+		$query =~ s/\{($params_re)}/\$$1/g;
 	}
 	
 	my $statement = [$query, $params // {}];
@@ -108,9 +114,14 @@ use Try::Tiny;
 sub _begin {
 	my ($self) = @_;
 	
-	croak 'Nested transactions unsupported in Bolt' if $self->{net}->{active_tx};
+	croak "Concurrent transactions are unsupported in Bolt (there is already an open transaction in this session)" if $self->{net}->{active_tx};
 	
-	$self->{bolt_txn} = $self->{net}->_new_tx;
+	try {
+		$self->{bolt_txn} = $self->{net}->_new_tx($self);
+	}
+	catch {
+		die $_ if $_ !~ m/\bprotocol version\b/i;  # Bolt v1/v2
+	};
 	$self->{net}->{active_tx} = 1;
 	$self->run('BEGIN') unless $self->{bolt_txn};
 	return $self;
@@ -120,7 +131,7 @@ sub _begin {
 sub _run_autocommit {
 	my ($self, $query, @parameters) = @_;
 	
-	croak 'Nested transactions unsupported in Bolt' if $self->{net}->{active_tx};
+	croak "Concurrent transactions are unsupported in Bolt (there is already an open transaction in this session)" if $self->{net}->{active_tx};
 	
 	$self->{net}->{active_tx} = 1;  # run() requires an active tx
 	my $results;
@@ -134,6 +145,7 @@ sub _run_autocommit {
 	$self->{net}->{active_tx} = 0;
 	
 	return $results unless wantarray;
+	warnings::warnif deprecated => "run() in list context is deprecated";
 	return $results->list if ref $query ne 'ARRAY';
 	return @$results;
 }
@@ -143,6 +155,7 @@ sub commit {
 	my ($self) = @_;
 	
 	croak 'Transaction already closed' unless $self->is_open;
+	croak 'Use `return` to commit a managed transaction' if $self->{managed};
 	
 	if ($self->{bolt_txn}) {
 		$self->{bolt_txn}->commit;
@@ -159,6 +172,7 @@ sub rollback {
 	my ($self) = @_;
 	
 	croak 'Transaction already closed' unless $self->is_open;
+	croak 'Explicit rollback of a managed transaction' if $self->{managed};
 	
 	if ($self->{bolt_txn}) {
 		$self->{bolt_txn}->rollback;
@@ -192,6 +206,19 @@ my $RESULT_DATA_CONTENTS = ['row', 'rest'];
 my $RESULT_DATA_CONTENTS_GRAPH = ['row', 'rest', 'graph'];
 
 
+sub _run_multiple {
+	my ($self, @statements) = @_;
+	
+	croak 'Transaction already closed' unless $self->is_open;
+	
+	return $self->{net}->_run( $self, map {
+		croak '_run_multiple() expects a list of array references' unless ref eq 'ARRAY';
+		croak '_run_multiple() with empty statements not allowed' unless $_->[0];
+		$self->_prepare(@$_);
+	} @statements );
+}
+
+
 sub _prepare {
 	my ($self, $query, @parameters) = @_;
 	
@@ -229,6 +256,8 @@ sub _run_autocommit {
 sub commit {
 	my ($self) = @_;
 	
+	croak 'Use `return` to commit a managed transaction' if $self->{managed};
+	
 	$self->_run_autocommit;
 }
 
@@ -237,6 +266,7 @@ sub rollback {
 	my ($self) = @_;
 	
 	croak 'Transaction already closed' unless $self->is_open;
+	croak 'Explicit rollback of a managed transaction' if $self->{managed};
 	
 	$self->{net}->_request($self, 'DELETE') if $self->{transaction_endpoint};
 	$self->{closed} = 1;
@@ -266,7 +296,7 @@ Neo4j::Driver::Transaction - Logical container for an atomic unit of work
 
 =head1 VERSION
 
-version 0.24
+version 0.37
 
 =head1 SYNOPSIS
 
@@ -302,6 +332,9 @@ execution of a statement to have completed, you need to use the
 L<Result|Neo4j::Driver::Result>, for example by calling
 one of the methods C<fetch()>, C<list()> or C<summary()>.
 
+Neo4j drivers allow the creation of different kinds of transactions.
+See L<Neo4j::Driver::Session> for details.
+
 =head1 METHODS
 
 L<Neo4j::Driver::Transaction> implements the following methods.
@@ -310,7 +343,7 @@ L<Neo4j::Driver::Transaction> implements the following methods.
 
  $transaction->commit;
 
-Commits the transaction and returns the result.
+Commits an unmanaged transaction.
 
 After committing the transaction is closed and can no longer be used.
 
@@ -329,7 +362,7 @@ S<60 seconds.>
 
  $transaction->rollback;
 
-Rollbacks the transaction.
+Rolls back a transaction.
 
 After rolling back the transaction is closed and can no longer be
 used.
@@ -399,17 +432,19 @@ converted to strings before they are sent to the Neo4j server.
 
 This driver always reports all errors using C<die()>. Error messages
 received from the Neo4j server are passed on as-is.
+See L<Neo4j::Driver::Plugin/"error"> for accessing error details.
 
 Statement errors occur when the statement is executed on the server.
 This may not necessarily have happened by the time C<run()> returns.
-If you use C<try> to handle errors, make sure you use the
-L<Result|Neo4j::Driver::Result> within the C<try> block, for example
-by calling one of the methods C<fetch()>, C<list()> or C<summary()>.
+If you use L<C<try>|perlsyn/"Try"> to handle errors, make sure
+you actually I<use> the L<Result|Neo4j::Driver::Result> within
+the C<try> block, for example by retrieving a record or calling
+the method C<has_next()>.
 
 Transactions are rolled back and closed automatically if the Neo4j
 server encounters an error when running a query. However, if an
 I<internal> error occurs in the driver or in one of its supporting
-modules, explicit transactions remain open.
+modules, unmanaged transactions may remain open.
 
 Typically, no particular handling of error conditions is required.
 But if you wrap your transaction in a C<try> (or C<eval>) block,
@@ -417,17 +452,17 @@ you intend to continue using the same session even after an error
 condition, I<and> you want to be absolutely sure the session is in
 a defined state, you can roll back a failed transaction manually:
 
- use Try::Tiny;
+ use feature 'try';
  $tx = $session->begin_transaction;
  try {
    ...;
    $tx->commit;
  }
- catch {
-   say "Database error: $_";
+ catch ($e) {
+   say "Database error: $e";
    ...;
    $tx->rollback if $tx->is_open;
- };
+ }
  # at this point, $session is safe to use
 
 =head1 EXPERIMENTAL FEATURES
@@ -437,60 +472,26 @@ features. These are subject to unannounced modification or removal
 in future versions. Expect your code to break if you depend upon
 these features.
 
-=head2 Calling in list context
-
- @records = $transaction->run('...');
- @results = $transaction->run([...]);
-
-The C<run> method tries to Do What You Mean if called in list
-context.
-
 =head2 Execute multiple statements at once
 
- $statements = [
+ @statements = (
    [ 'RETURN 42' ],
    [ 'RETURN {value}', value => 'forty-two' ],
- ];
- $results = $transaction->run($statements);
- foreach $result ( @$results ) {
+ );
+ @results = $transaction->_run_multiple(@statements);
+ foreach $result ( @results ) {
    say $result->single->get;
  }
 
 The Neo4j HTTP API supports executing multiple statements within a
 single HTTP request. This driver exposes this feature to the client.
+It is only available on HTTP connections.
 
-This feature is likely to be removed from this driver in favour of
-lazy execution, similar to the official Neo4j drivers.
-
-=head2 Disable obtaining query statistics
-
- $transaction = $session->begin_transaction;
- $transaction->{return_stats} = 0;
- $result = $transaction->run('...');
-
-Since version 0.13, this driver requests query statistics from the
-Neo4j server by default. When using HTTP, this behaviour can be
-disabled. Doing so might provide a very minor performance increase.
-
-The ability to disable the statistics may be removed in future.
-
-=head2 Return results in graph format
-
- $session = $driver->config( jolt => 0 )->session;
- $transaction = $session->begin_transaction;
- $transaction->{return_graph} = 1;
- $records = $transaction->run('...')->list;
- for $record ( @$records ) {
-   $graph_data = $record->{graph};
-   ...
- }
-
-The Neo4j HTTP JSON API supports a "graph" results data format.
-This feature is only available when Jolt is disabled, which is
-not recommended. The C<return_graph> option will most likely be
-removed entirely in future versions, as requesting the "graph"
-result is also possible by using a custom networking module;
-see L<Neo4j::Driver::Net>.
+This feature might eventually be used to implement lazy statement
+execution for this driver. The private C<_run_multiple()> method
+which makes using this feature explicit is expected to remain
+available at least until that time. See also
+L<Neo4j::Driver::Plugin/"USE OF INTERNAL APIS">.
 
 =head1 SEE ALSO
 
@@ -501,12 +502,14 @@ see L<Neo4j::Driver::Net>.
 =item * L<Neo4j::Driver::B<Result>>
 
 =item * Equivalent documentation for the official Neo4j drivers:
-L<Transaction (Java)|https://neo4j.com/docs/api/java-driver/current/index.html?org/neo4j/driver/Transaction.html>,
-L<Transaction (JavaScript)|https://neo4j.com/docs/api/javascript-driver/current/class/src/transaction.js~Transaction.html>,
-L<ITransaction (.NET)|https://neo4j.com/docs/api/dotnet-driver/4.0/html/b64c7dfe-87e9-8b85-5a02-8ff03800b67b.htm>,
-L<Sessions & Transactions (Python)|https://neo4j.com/docs/api/python-driver/current/api.html#transaction>
+L<Transaction (Java)|https://neo4j.com/docs/api/java-driver/5.2/org.neo4j.driver/org/neo4j/driver/Transaction.html>,
+L<Transaction (JavaScript)|https://neo4j.com/docs/api/javascript-driver/5.2/class/lib6/transaction.js~Transaction.html>,
+L<ITransaction (.NET)|https://neo4j.com/docs/api/dotnet-driver/5.2/html/b64c7dfe-87e9-8b85-5a02-8ff03800b67b.htm>,
+L<Sessions & Transactions (Python)|https://neo4j.com/docs/api/python-driver/5.2/api.html#transaction>
 
-=item * Neo4j L<Transactional Cypher HTTP API|https://neo4j.com/docs/developer-manual/3.0/http-api/>
+=item * Neo4j L<Transactional Cypher HTTP API|https://neo4j.com/docs/http-api/5/actions/>
+
+=item * L<Feature::Compat::Try>
 
 =back
 
@@ -514,12 +517,15 @@ L<Sessions & Transactions (Python)|https://neo4j.com/docs/api/python-driver/curr
 
 Arne Johannessen <ajnn@cpan.org>
 
+If you contact me by email, please make sure you include the word
+"Perl" in your subject header to help beat the spam filters.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
+This software is Copyright (c) 2016-2023 by Arne Johannessen.
 
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
+This is free software; you can redistribute it and/or modify it under
+the terms of the Artistic License 2.0 or (at your option) the same terms
+as the Perl 5 programming language system itself.
 
 =cut
