@@ -5,7 +5,7 @@ use utf8;
 
 package Neo4j::Driver::Type::Path;
 # ABSTRACT: Directed sequence of relationships between two nodes
-$Neo4j::Driver::Type::Path::VERSION = '0.24'; # TRIAL
+$Neo4j::Driver::Type::Path::VERSION = '0.37'; # TRIAL
 
 use parent 'Neo4j::Types::Path';
 use overload '@{}' => \&_array, fallback => 1;
@@ -16,7 +16,6 @@ use Carp qw(croak);
 sub nodes {
 	my ($self) = @_;
 	
-	croak 'nodes() in scalar context not supported' unless wantarray;
 	my $i = 0;
 	return grep { ++$i & 1 } @{$self->{path}};
 }
@@ -25,7 +24,6 @@ sub nodes {
 sub relationships {
 	my ($self) = @_;
 	
-	croak 'relationships() in scalar context not supported' unless wantarray;
 	my $i = 0;
 	return grep { $i++ & 1 } @{$self->{path}};
 }
@@ -34,12 +32,12 @@ sub relationships {
 sub elements {
 	my ($self) = @_;
 	
-	croak 'elements() in scalar context not supported' unless wantarray;
 	return @{$self->{path}};
 }
 
 
 sub path {
+	# uncoverable pod (see Deprecations.pod)
 	my ($self) = @_;
 	
 	warnings::warnif deprecated => __PACKAGE__ . "->path() is deprecated; use elements()";
@@ -78,7 +76,7 @@ Neo4j::Driver::Type::Path - Directed sequence of relationships between two nodes
 
 =head1 VERSION
 
-version 0.24
+version 0.37
 
 =head1 SYNOPSIS
 
@@ -131,12 +129,14 @@ these features.
 
 =head2 Calling in scalar context
 
- $all   = $path->elements;  # fails
- $nodes = $path->nodes;  # fails
- $rels  = $path->relationships;  # fails
+ $count = $path->elements;
+ $count = $path->nodes;
+ $count = $path->relationships;
 
 The C<elements()>, C<nodes()>, and C<relationships()> methods
-C<die> if called in scalar context.
+return the number of items if called in scalar context.
+
+Until version 0.25, they C<die>d instead.
 
 =head1 BUGS
 
@@ -153,7 +153,7 @@ their labels and types. This is due to an issue in the Neo4j server.
 =item * L<Neo4j::Types::Path>
 
 =item * Equivalent documentation for the official Neo4j drivers:
-L<Path (Java)|https://neo4j.com/docs/api/java-driver/current/index.html?org/neo4j/driver/types/Path.html>
+L<Path (Java)|https://neo4j.com/docs/api/java-driver/5.2/org.neo4j.driver/org/neo4j/driver/types/Path.html>
 
 =back
 
@@ -161,12 +161,15 @@ L<Path (Java)|https://neo4j.com/docs/api/java-driver/current/index.html?org/neo4
 
 Arne Johannessen <ajnn@cpan.org>
 
+If you contact me by email, please make sure you include the word
+"Perl" in your subject header to help beat the spam filters.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
+This software is Copyright (c) 2016-2023 by Arne Johannessen.
 
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
+This is free software; you can redistribute it and/or modify it under
+the terms of the Artistic License 2.0 or (at your option) the same terms
+as the Perl 5 programming language system itself.
 
 =cut
