@@ -5,7 +5,7 @@ use utf8;
 
 package Neo4j::Driver::Type::Temporal;
 # ABSTRACT: Represents a Neo4j temporal value
-$Neo4j::Driver::Type::Temporal::VERSION = '0.24'; # TRIAL
+$Neo4j::Driver::Type::Temporal::VERSION = '0.37'; # TRIAL
 
 # may not be supported by Bolt
 
@@ -23,7 +23,7 @@ Neo4j::Driver::Type::Temporal - Represents a Neo4j temporal value
 
 =head1 VERSION
 
-version 0.24
+version 0.37
 
 =head1 DESCRIPTION
 
@@ -48,7 +48,7 @@ which L<Neo4j::Bolt> depends on internally. Use HTTP instead.
 
 =item * L<Neo4j::Driver>
 
-=item * L<"Temporal values" in Neo4j Cypher Manual|https://neo4j.com/docs/cypher-manual/current/syntax/temporal/>
+=item * L<"Temporal values" in Neo4j Cypher Manual|https://neo4j.com/docs/cypher-manual/5/syntax/temporal/>
 
 =back
 
@@ -56,12 +56,15 @@ which L<Neo4j::Bolt> depends on internally. Use HTTP instead.
 
 Arne Johannessen <ajnn@cpan.org>
 
+If you contact me by email, please make sure you include the word
+"Perl" in your subject header to help beat the spam filters.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016-2021 by Arne Johannessen.
+This software is Copyright (c) 2016-2023 by Arne Johannessen.
 
-This is free software, licensed under:
-
-  The Artistic License 2.0 (GPL Compatible)
+This is free software; you can redistribute it and/or modify it under
+the terms of the Artistic License 2.0 or (at your option) the same terms
+as the Perl 5 programming language system itself.
 
 =cut
